-- -------------------------------------------------------------------------- --
--           _    _ ___   __           ____            _____      _           --
--          / \  | | \ \ / /__  _   _ / ___|__ _ _ __ | ____|__ _| |_         --
--         / _ \ | | |\ V / _ \| | | | |   / _` | '_ \|  _| / _` | __|        --
--        / ___ \| | | | | (_) | |_| | |__| (_| | | | | |__| (_| | |_         --
--       /_/   \_\_|_| |_|\___/ \__,_|\____\__,_|_| |_|_____\__,_|\__|        --
--                                                                            --
-- -------------------------------------------------------------------------- --

-- -------------------------------------------------------------------------- --
-- AllYouCanEat by kunda                                                      --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- AllYouCanEat is digging for details about the following statistics:        --
-- - 1774 = Different beverages consumed                                      --
-- - 1775 = Different foods eaten                                             --
-- - 1298 = Different bandage types used                                      --
-- - 1300 = Different health potions used                                     --
-- - 1302 = Different mana potions used                                       --
-- - 1304 = Different elixirs used                                            --
-- - 1306 = Different flasks consumed                                         --
--                                                                            --
-- With this AddOn it's easy to see which foods, beverages, elixirs, potions, --
-- bandages and flasks were consumed and which not.                           --
-- The default statistic interface does not show such details.                --
--                                                                            --
-- The result is shown in a movable frame (accessible via InterfaceOptions,   --
-- LDB or '/allyoucaneat'). If the statistic data is saved to SavedVariables  --
-- (via 'CharManager' Button) it's possible to enable an option to see this   --
-- information in the various Tooltips.                                       --
--                                                                            --
-- !   It's impossible to consume everything because some items are class   ! --
-- !   dependent, profession dependent, faction dependent, quest rewards    ! --
-- !   (which you usually not get back if you finished the quest), some     ! --
-- !   items are only available via a TCG NPC and finally some items are    ! --
-- !   simply not available ingame.                                         ! --
--                                                                            --
-- !!!  YES, this AddOn is absolutely USELESS, unless it's fun for you to !!! --
-- !!!  do a drink & food hunt across the whole WoW world :)              !!! --
--                                                                            --
-- Notes:                                                                     --
-- -> The initial statistic scan at PLAYER_LOGIN is very CPU/Bandwidth heavy  --
--    (approximately 1700 GetAchievementCriteriaInfo() calls).                --
--    This scan can take up to 2 or 3 second(s)!!!                            --
-- -> The Tooltip server query code may be a bit annoying, but this is        --
--    necessary to avoid disconnects, because some itemIDs that are returned  --
--    from the server in the initial statistic scan are invalid respectively  --
--    currently not available ingame.                                         --
--    This means: A Tooltip server query (mouseover an item in the MainFrame) --
--    via GameTooltip:SetHyperlink() is restricted to one server query every  --
--    5 seconds, for the same itemID to 10 seconds. I never got a disconnect  --
--    with this 2 intervals. But just in case: A warning text is shown.       --
--                                                                            --
-- -------------------------------------------------------------------------- --

-- ---------------------------------------------------------------------------------------------------------------------
AllYouCanEat = CreateFrame("Frame")
AllYouCanEat_Options = {}      -- SavedVariable options table
AllYouCanEat_Data = {}         -- SavedVariable data table
local L = AllYouCanEat_Locales -- localization table

local _G = _G
local select = _G.select
local GetTime = _G.GetTime
local GetAchievementCriteriaInfo = _G.GetAchievementCriteriaInfo

local mult2 = 10^(2 or 0)      -- x.xx

local playerRealm              -- current realm name
local playerName               -- current player name
local playerGUID               -- current player GUID
local playerFaction            -- current player faction
local playerClass              -- current player class

local classcolors = RAID_CLASS_COLORS

local DATA = {}                -- main data table
local GUI_Data = {}            -- data table
local GUI_Data_Count           -- data table count
local GUI_SortBy = 1           -- data table sort by: 1 = number | 2 = itemId | 3 = itemLink
local GUI_SortOrder = 1        -- data table sort order
local GUI_Detail = 1           -- data table detail: 1 = all | 2 = consumed | 3 = not consumed
local GUI_Data_maxNum = 30     -- data table max rows

local GUI_Selection = 1774     -- default data
local GUI_scantime = ""        -- latest update

local GUI_CharListData = {}         -- digged character table
local GUI_CharListData_maxNum = 10  -- digged character table max rows
local GUI_RealmListData = {}        -- digged realm table
local GUI_RealmListData_Count       -- digged realm table count
local GUI_RealmListData_maxNum = 10 -- digged realm table max rows

local StatisticID = {
	[1] = {id = 1774, name = L["Beverages"],      icon = "Interface\\Icons\\INV_Drink_15"},        -- 1774 = Different beverages consumed
	[2] = {id = 1775, name = L["Foods"],          icon = "Interface\\Icons\\INV_Misc_Food_08"},    -- 1775 = Different foods eaten
	[3] = {id = 1298, name = L["Bandage"],        icon = "Interface\\Icons\\INV_Misc_Bandage_14"}, -- 1298 = Different bandage types used
	[4] = {id = 1300, name = L["Health Potions"], icon = "Interface\\Icons\\INV_Potion_51"},       -- 1300 = Different health potions used
	[5] = {id = 1302, name = L["Mana Potions"],   icon = "Interface\\Icons\\INV_Potion_72"},       -- 1302 = Different mana potions used
	[6] = {id = 1304, name = L["Elixirs"],        icon = "Interface\\Icons\\INV_Potion_66"},       -- 1304 = Different elixirs used
	[7] = {id = 1306, name = L["Flasks"],         icon = "Interface\\Icons\\INV_Potion_90"}        -- 1306 = Different flasks consumed
}

local TooltipCodeIsNeeded
local ServerQueryTime = {}            -- checker for Tooltip server query in function AllYouCanEat:ShowTooltip() to avoid disconnects
local ServerQueryTimeLast = GetTime() -- checker for Tooltip server query in function AllYouCanEat:ShowTooltip() to avoid disconnects

local criteriaTimer = ServerQueryTimeLast
local UPDATE_itemID -- value for CRITERIA_UPDATE
local UPDATE_statID -- value for CRITERIA_UPDATE
local UPDATE_catID  -- value for CRITERIA_UPDATE

local Hooks = {}
function Hooks:SetAction(id)
	local _, item = self:GetItem()
	if not item then return end
	AllYouCanEat:AddNote(self, item)
end
function Hooks:SetAuctionItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetAuctionSellItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetBagItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetHyperlink()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetInboxItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetInventoryItem(unit, slot)
	if type(slot) ~= "number" or slot < 0 then return end
	AllYouCanEat:AddNote(self)
end
function Hooks:SetLootItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetLootRollItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetMerchantCostItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetMerchantItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetQuestItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetQuestLogItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetSendMailItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetTradePlayerItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetTradeSkillItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetTradeTargetItem()
	AllYouCanEat:AddNote(self)
end
function Hooks:SetGuildBankItem()
	AllYouCanEat:AddNote(self)
end

local function InstallHooks(tooltip, hooks)
	for name, func in pairs(hooks) do
		hooksecurefunc(tooltip, name, func)
	end
end

local function UpdateTime()
	local hour, minute = GetGameTime()
	if hour < 10 then
		hour = "0"..hour
	end
	if minute < 10 then
		minute = "0"..minute
	end
	GUI_scantime = hour..":"..minute
end

local function SetTabButton(name, showIt)
	if showIt then
		_G[name.."Bottom"]:SetTexture(0, 0, 0, 1)
		_G[name.."Border"]:SetTexture(0.4, 0.4, 0.4, 1)
	else
		_G[name.."Bottom"]:SetTexture(0.4, 0.4, 0.4, 1)
		_G[name.."Border"]:SetTexture(0.4, 0.4, 0.4, 0.4)
	end
end

local function ClassHexColor(class)
	local hex = string.format("%.2x%.2x%.2x", classcolors[class].r*255, classcolors[class].g*255, classcolors[class].b*255)
	return hex or "cccccc"
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:InitOptions()
	SlashCmdList["ALLYOUCANEAT"] = function() AllYouCanEat:MainFrameToggle() end
	SLASH_ALLYOUCANEAT1 = "/allyoucaneat"

	if AllYouCanEat_Options.version == nil then AllYouCanEat_Options.version = 1 end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:LDBcheck()
	if LibStub and LibStub:GetLibrary("CallbackHandler-1.0", true) and LibStub:GetLibrary("LibDataBroker-1.1", true) then
		LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("AllYouCanEat", {
			type = "launcher",
			icon = "Interface\\Minimap\\Tracking\\Food",
			OnClick = function(self, button)
				AllYouCanEat:MainFrameToggle()
			end,
		})
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:CreateInterfaceOptions()
	local options = CreateFrame("Frame", "AllYouCanEat_InterfaceOptions")
	options.name = "AllYouCanEat"

	local title = options:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetPoint("TOPLEFT", 16, -16)
	title:SetText("AllYouCanEat")
	title:SetJustifyH("LEFT")
	title:SetJustifyV("TOP")

	CreateFrame("Button", "AllYouCanEat_InterfaceOptions_GUI", AllYouCanEat_InterfaceOptions, "UIPanelButtonTemplate")
	AllYouCanEat_InterfaceOptions_GUI:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -10)
	AllYouCanEat_InterfaceOptions_GUI:SetWidth(120)
	AllYouCanEat_InterfaceOptions_GUI:SetHeight(24)
	AllYouCanEat_InterfaceOptions_GUIText:SetText(L["Open GUI"])
	AllYouCanEat_InterfaceOptions_GUI:SetScript("OnClick", function(self)
		InterfaceOptionsFrame_Show()   -- hide InterfaceOptionsFrame
		HideUIPanel(GameMenuFrame)     -- hide GameMenuFrame
		AllYouCanEat:MainFrameToggle() -- show GUI
	end)

	InterfaceOptions_AddCategory(options)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:InitialStatisticScan()
	--local timerStart = GetTime() -- timecheck

	for i = 1, #StatisticID do
		local statID = StatisticID[i].id
		local total = 0
		local totalcompleted = 0

		if not DATA[statID] then DATA[statID] = {} end

		for j = 1, 1000 do	
			local itemName, _, _, _, _, _, _, itemID, _, criteriaID = GetAchievementCriteriaInfo(statID, j)
			if not itemID then break end
			local _, _, isCompleted = GetAchievementCriteriaInfo(criteriaID)

			if not DATA[statID][itemID] then
				DATA[statID][itemID] = {}
				DATA[statID][itemID].itemname   = itemName
				DATA[statID][itemID].criteriaID = criteriaID
				DATA[statID][itemID].completed  = isCompleted

				total = total + 1

				if isCompleted then
					totalcompleted = totalcompleted + 1
				end
			end

		end

		DATA[statID].total = total
		DATA[statID].totalcompleted = totalcompleted
	end

	UpdateTime()
	--print("timecheck:", GetTime()-timerStart) --timecheck
end

function AllYouCanEat:SaveCharacterData()
	if not AllYouCanEat_Data.chars              then AllYouCanEat_Data.chars = {} end
	if not AllYouCanEat_Data.chars[playerRealm] then AllYouCanEat_Data.chars[playerRealm] = {}	end
	if not AllYouCanEat_Data.chars[playerRealm][playerGUID] then
		AllYouCanEat_Data.chars[playerRealm][playerGUID] = {}
		AllYouCanEat_Data.chars[playerRealm][playerGUID].name = playerName
		AllYouCanEat_Data.chars[playerRealm][playerGUID].class = playerClass
		AllYouCanEat_Data.chars[playerRealm][playerGUID].faction = playerFaction
		AllYouCanEat_Data.chars[playerRealm][playerGUID].savestats = false
		AllYouCanEat_Data.chars[playerRealm][playerGUID].tooltip = false
	else
		-- check if name, class or faction has changed
		if AllYouCanEat_Data.chars[playerRealm][playerGUID].name ~= playerName or
		   AllYouCanEat_Data.chars[playerRealm][playerGUID].class ~= playerClass or
		   AllYouCanEat_Data.chars[playerRealm][playerGUID].faction ~= playerFaction
		then
			AllYouCanEat_Data.chars[playerRealm][playerGUID].name = playerName
			AllYouCanEat_Data.chars[playerRealm][playerGUID].class = playerClass
			AllYouCanEat_Data.chars[playerRealm][playerGUID].faction = playerFaction
		end
	end
end

function AllYouCanEat:SaveCharacterStatisticData()
	if not AllYouCanEat_Data.stats then AllYouCanEat_Data.stats = {} end

	for statID, data1 in pairs(DATA) do
		if not AllYouCanEat_Data.stats[statID] then AllYouCanEat_Data.stats[statID] = {} end
		for itemID, data2 in pairs(data1) do
			if itemID ~= "total" and itemID ~= "totalcompleted" then
				if not data2.completed then
					if not AllYouCanEat_Data.stats[statID][itemID] then AllYouCanEat_Data.stats[statID][itemID] = {} end
					AllYouCanEat_Data.stats[statID][itemID][playerGUID] = true
				end
			end
		end
	end

	AllYouCanEat_Data.chars[playerRealm][playerGUID].savestats = true
end

function AllYouCanEat:DeleteCharacterStatisticData(guid)
	for statID, data1 in pairs(AllYouCanEat_Data.stats) do
		for itemID, data2 in pairs(data1) do
			if AllYouCanEat_Data.stats[statID][itemID][guid] then
				AllYouCanEat_Data.stats[statID][itemID][guid] = nil
			end

			local x = 0
			for k, v in pairs(AllYouCanEat_Data.stats[statID][itemID]) do
				x = x + 1
				break
			end
			if x == 0 then
				AllYouCanEat_Data.stats[statID][itemID] = nil
			end
		end

		local x = 0
		for k, v in pairs(AllYouCanEat_Data.stats[statID]) do
			x = x + 1
			break
		end
		if x == 0 then
			AllYouCanEat_Data.stats[statID] = nil
		end
	end

	local x = 0
	for k, v in pairs(AllYouCanEat_Data.stats) do
		x = x + 1
		break
	end
	if x == 0 then
		AllYouCanEat_Data.stats = nil
	end

	AllYouCanEat_Data.chars[playerRealm][guid].savestats = false
	AllYouCanEat_Data.chars[playerRealm][guid].tooltip = false
end

function AllYouCanEat:CheckIfTooltipCodeIsNeeded()
	local x = 0
	for playerguid, data in pairs(AllYouCanEat_Data.chars[playerRealm]) do
		if data.tooltip == true then
			x = x + 1
			break
		end
	end
	
	if x > 0 then
		TooltipCodeIsNeeded = true
	else
		TooltipCodeIsNeeded = false
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:TooltipInfoText(itemID, timeElapsed)
	GameTooltip:ClearLines()
	GameTooltip:AddLine("|TInterface\\DialogFrame\\UI-Dialog-Icon-AlertNew:24|t "..L["Unsafe Item"], 1, 0, 0)
	GameTooltip:AddLine(L["ItemID"]..": "..itemID)
	if timeElapsed then
		timeElapsed = math.floor((10 - timeElapsed) * mult2 + 0.5) / mult2
		GameTooltip:AddLine(string.format(L["You can try to start a new server query in %s seconds!"], "|cffffff7f"..timeElapsed.."|r"), 1, 1, 1, 1)
		GameTooltip:AddLine(L["You may be disconnected."], 1, 0.5, 0, 1)
	else
		GameTooltip:AddLine(L["Please wait 5 seconds for next server query."], 1, 1, 1, 1)
	end
end

function AllYouCanEat:ShowTooltip(self, itemID)
	local _, itemLink = GetItemInfo(itemID)

	GameTooltip:SetOwner(AllYouCanEat_DataFrame, "ANCHOR_BOTTOMRIGHT", 0, 382)

	if itemLink then
		ServerQueryTime[itemID] = nil
		GameTooltip:SetHyperlink(itemLink)
		GameTooltip:Show()
	else
		if ServerQueryTime[itemID] then
			local timeElapsed = GetTime() - ServerQueryTime[itemID]
			if timeElapsed > 10 then
				ServerQueryTime[itemID] = nil
			else
				AllYouCanEat:TooltipInfoText(itemID, timeElapsed)
				GameTooltip:Show()
				return
			end
		else
			ServerQueryTime[itemID] = GetTime()
		end

		local QueryIsSave
		if GetTime() - ServerQueryTimeLast > 5 then
			QueryIsSave = true
		end

		if QueryIsSave then
			ServerQueryTimeLast = GetTime()
			GameTooltip:SetHyperlink("item:"..itemID)
			AllYouCanEat:TooltipInfoText(itemID, 0)
		else
			AllYouCanEat:TooltipInfoText(itemID)
		end
		GameTooltip:Show()
	end
end

function AllYouCanEat:AddNote(tooltip, item)
	--local timerStart = GetTime() -- timecheck
	if not TooltipCodeIsNeeded then return end
	local item = item or select(2, tooltip:GetItem())
	if not item then return end
	local itemID = select(3, item:find("item:(%d+)"))
	if not itemID then return end

	itemID = tonumber(itemID)

	local hasID
	local chars = {}

	for i = 1, #StatisticID do
		local statID = StatisticID[i].id
		if DATA[statID][itemID] then
			hasID = true
			if AllYouCanEat_Data.stats and
			   AllYouCanEat_Data.stats[statID] and
			   AllYouCanEat_Data.stats[statID][itemID]
			then
				local x = 1
				for guid in pairs(AllYouCanEat_Data.stats[statID][itemID]) do
					if AllYouCanEat_Data.chars[playerRealm][guid] and
					   AllYouCanEat_Data.chars[playerRealm][guid].savestats and
					   AllYouCanEat_Data.chars[playerRealm][guid].tooltip
					then
						chars[x] = AllYouCanEat_Data.chars[playerRealm][guid].name
						x = x + 1
					end
				end
			end
		end
	end

	if not hasID then return end

	local upval = true

	for guid, data in pairs(AllYouCanEat_Data.chars[playerRealm]) do
		if data.savestats and data.tooltip then

			local notfinish
			for i = 1, #chars do
				if chars[i] == data.name then
					notfinish = true
				end				
			end

			if upval then
				tooltip:AddLine("AllYouCanEat:", 1, 0.5, 0)
				upval = false
			end

			if notfinish then
				tooltip:AddDoubleLine("|cff"..ClassHexColor(data.class)..data.name.."|r", L["not consumed"], 1, 1, 1,  1, 0, 0)
			else
				tooltip:AddDoubleLine("|cff"..ClassHexColor(data.class)..data.name.."|r", L["consumed"], 1, 1, 1,  0, 0.75, 0)
			end

		end
	end

	if tooltip:IsShown() then tooltip:Show() end
	--print("timecheck:", GetTime()-timerStart) --timecheck
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function TemplateBorderTRBL(frame)
	local texture = frame:CreateTexture(nil, "BORDER")
	texture:SetPoint("TOPLEFT", 1, -1)
	texture:SetPoint("BOTTOMRIGHT", -1, 1)
	texture:SetTexture(0, 0, 0, 1)
	local textureBackground = frame:CreateTexture(nil, "BACKGROUND")
	textureBackground:SetPoint("TOPLEFT", 0, 0)
	textureBackground:SetPoint("BOTTOMRIGHT", 0, 0)
	textureBackground:SetTexture(0.4,  0.4,  0.4,  1)
end

local function TemplateHeadlineButton(button, text, cut)
	local name = button:GetName()
	local texture = button:CreateTexture(nil, "BORDER")
	texture:SetPoint("TOPLEFT", 1, -1)
	texture:SetPoint("BOTTOMRIGHT", -1, 1)
	texture:SetTexture(0, 0, 0, 1)

	local textureBackground = button:CreateTexture(nil, "BACKGROUND")
	textureBackground:SetPoint("TOPLEFT", 0, 0)
	textureBackground:SetPoint("BOTTOMRIGHT", 0, 0)
	textureBackground:SetTexture(0.4, 0.4, 0.4, 1)

	if cut == 1 then
		local textureNormal = button:CreateTexture(nil, "ARTWORK")
		textureNormal:SetPoint("TOPLEFT", 3, -3)
		textureNormal:SetPoint("BOTTOMRIGHT", -3, 3)
		textureNormal:SetTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
		textureNormal:SetTexCoord(8/32, 22/32, 10/32, 22/32)
		button:SetNormalTexture(textureNormal)
		local texturePush = button:CreateTexture(nil, "ARTWORK")
		texturePush:SetPoint("TOPLEFT", 4, -4)
		texturePush:SetPoint("BOTTOMRIGHT", -4, 4)
		texturePush:SetTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
		texturePush:SetTexCoord(8/32, 22/32, 10/32, 22/32)
		button:SetPushedTexture(texturePush)
	elseif cut == 2 then
		local textureNormal = button:CreateTexture(name.."Background", "ARTWORK")
		textureNormal:SetPoint("TOPLEFT", 3, -3)
		textureNormal:SetPoint("BOTTOMRIGHT", -3, 3)
		textureNormal:SetTexture(1, 1, 1, 0)
		button:SetNormalTexture(textureNormal)
	elseif cut == 3 then
		local textureNormal = button:CreateTexture(nil, "ARTWORK")
		textureNormal:SetPoint("TOPLEFT", 3, -3)
		textureNormal:SetPoint("BOTTOMRIGHT", -3, 3)
		textureNormal:SetTexture("Interface\\Minimap\\Tracking\\None")
		textureNormal:SetTexCoord(3/32, 31/32, 1/32, 29/32)
		button:SetNormalTexture(textureNormal)
		local texturePush = button:CreateTexture(nil, "ARTWORK")
		texturePush:SetPoint("TOPLEFT", 4, -4)
		texturePush:SetPoint("BOTTOMRIGHT", -4, 4)
		texturePush:SetTexture("Interface\\Minimap\\Tracking\\None")
		texturePush:SetTexCoord(3/32, 31/32, 1/32, 29/32)
		button:SetPushedTexture(texturePush)
	end

	local textureHighlight = button:CreateTexture(nil, "ARTWORK")
	textureHighlight:SetPoint("TOPLEFT", 3, -3)
	textureHighlight:SetPoint("BOTTOMRIGHT", -3, 3)
	textureHighlight:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetHighlightTexture(textureHighlight)

	if text then
		local buttontext = button:CreateFontString(name.."Text", "OVERLAY", "GameFontWhiteSmall")
		buttontext:SetText(text)
		buttontext:SetWidth( buttontext:GetStringWidth()+10 )
		buttontext:SetHeight(12)
		buttontext:SetPoint("CENTER", button, "CENTER", 0, 0)
		buttontext:SetJustifyH("CENTER")
		buttontext:SetTextColor(1, 1, 1, 1)
	end
end

local function TemplateTabButton(button, text, detail)
	local name = button:GetName()
	local texture = button:CreateTexture(nil, "BORDER")
	texture:SetPoint("TOPLEFT", 1, -1)
	texture:SetPoint("BOTTOMRIGHT", -1, 1)
	texture:SetTexture(0, 0, 0, 1)

	local textureBorder = button:CreateTexture(name.."Border", "BACKGROUND")
	textureBorder:SetPoint("TOPLEFT", 0, 0)
	textureBorder:SetPoint("BOTTOMRIGHT", -1, 1)
	textureBorder:SetPoint("TOPRIGHT"    ,0, 0)
	textureBorder:SetPoint("BOTTOMLEFT" ,1, 1)
	textureBorder:SetTexture(0.4, 0.4, 0.4, 1)

	local textureBottom = button:CreateTexture(name.."Bottom", "ARTWORK")
	textureBottom:SetPoint("TOPLEFT", button, "BOTTOMLEFT" ,1, 2)
	textureBottom:SetPoint("BOTTOMLEFT"  ,1, 1)
	textureBottom:SetPoint("TOPRIGHT", button, "BOTTOMRIGHT" ,-1, 2)
	textureBottom:SetPoint("BOTTOMRIGHT" ,-1, 1)
	textureBottom:SetTexture(0.4, 0.4, 0.4, 1)

	local textureHighlight = button:CreateTexture(nil, "ARTWORK")
	textureHighlight:SetPoint("TOPLEFT", 3, -3)
	textureHighlight:SetPoint("BOTTOMRIGHT", -3, 3)
	textureHighlight:SetTexture(1, 1, 1, 0.2)
	button:SetHighlightTexture(textureHighlight)

	local buttontext = button:CreateFontString(name.."Text", "OVERLAY", "GameFontNormalSmall")
	buttontext:SetText(text)
	buttontext:SetWidth( buttontext:GetStringWidth()+10 )
	buttontext:SetHeight(12)
	buttontext:SetPoint("CENTER", button, "CENTER", 0, 0)
	buttontext:SetJustifyH("CENTER")

	if detail == 1 then
		buttontext:SetTextColor(1, 1, 1, 1)
	elseif detail == 2 then
		buttontext:SetTextColor(0, 0.75, 0, 1)
	elseif detail == 3 then
		buttontext:SetTextColor(1, 0, 0, 1)
	end
end

local function TemplateCheckButton(button)
	local textureNormal = button:CreateTexture(nil, "ARTWORK")
	textureNormal:SetPoint("TOPLEFT", 0, 0)
	textureNormal:SetPoint("BOTTOMRIGHT", 0, 0)
	textureNormal:SetPoint("CENTER", 0, 0)
	textureNormal:SetTexture("Interface\\Buttons\\UI-CheckBox-Up")
	textureNormal:SetTexCoord(4/32, 27/32, 5/32, 26/32)
	button:SetNormalTexture(textureNormal)

	local textureHighlight = button:CreateTexture(nil, "ARTWORK")
	textureHighlight:SetPoint("TOPLEFT", 1, -1)
	textureHighlight:SetPoint("BOTTOMRIGHT", -1, 1)
	textureHighlight:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetHighlightTexture(textureHighlight)

	local texturePush = button:CreateTexture(nil, "ARTWORK")
	texturePush:SetPoint("TOPLEFT", 0, 0)
	texturePush:SetPoint("BOTTOMRIGHT", 0, 0)
	texturePush:SetTexture("Interface\\Buttons\\UI-CheckBox-Down")
	texturePush:SetTexCoord(4/32, 27/32, 5/32, 26/32)
	button:SetPushedTexture(texturePush)

	local textureChecked = button:CreateTexture(nil, "ARTWORK")
	textureChecked:SetWidth(16)
	textureChecked:SetHeight(16)
	textureChecked:SetPoint("CENTER", 0, 0)
	textureChecked:SetTexture("Interface\\Buttons\\UI-CheckBox-Check")
	button:SetCheckedTexture(textureChecked)
end

local function TemplateDataListSortButton(button, text, pos, width)
	local name = button:GetName()

	local textureBackground = button:CreateTexture(nil, "BACKGROUND")
	textureBackground:SetPoint("TOPLEFT", 0, 0)
	textureBackground:SetPoint("BOTTOMRIGHT", 0, 0)
	textureBackground:SetTexture(1, 1, 1, 0.1)

	local buttontext
	if text then
		if pos == "right" then
			buttontext = button:CreateFontString(name.."Text", "OVERLAY", "GameFontNormalSmall")
			buttontext:SetText(text)
			local fw = buttontext:GetStringWidth()
			if fw >= width then
				buttontext:SetWidth( width-10 )
			end
			buttontext:SetHeight(12)
			buttontext:SetPoint("RIGHT", button, "RIGHT", 0, 0)
			buttontext:SetJustifyH("RIGHT")
		else
			buttontext = button:CreateFontString(name.."Text", "OVERLAY", "GameFontNormalSmall")
			buttontext:SetText(text)
			local fw = buttontext:GetStringWidth()
			if fw >= width then
				buttontext:SetWidth( width-10 )
			end
			buttontext:SetHeight(12)
			buttontext:SetPoint("LEFT", button, "LEFT", 0, 0)
			buttontext:SetJustifyH("LEFT")
		end
	end

	local texture = button:CreateTexture(name.."Arrow", "BACKGROUND")
	texture:SetWidth(7)
	texture:SetHeight(8)
	texture:SetTexture("Interface\\Buttons\\UI-SortArrow")
	if text then
		if pos == "right" then
			texture:SetPoint("RIGHT", buttontext, "LEFT", -1, 0)
		else
			texture:SetPoint("LEFT", buttontext, "RIGHT", 1, 0)
		end
	else
		texture:SetPoint("CENTER", button, "CENTER", 0, 0)
	end

	local textureHighlight = button:CreateTexture(nil, "ARTWORK")
	textureHighlight:SetPoint("TOPLEFT", 0, 0)
	textureHighlight:SetPoint("BOTTOMRIGHT", 0, 0)
	textureHighlight:SetTexture(1, 1, 1, 0.2)
	button:SetHighlightTexture(textureHighlight)
end

local function TemplateDataList(button, isSort)
	local name = button:GetName()

	if isSort then
		local sort_number = CreateFrame("Button", name.."Button_Number", button)
		TemplateDataListSortButton(sort_number, L["No."], "right", 40)
		sort_number:SetWidth(40)
		sort_number:SetHeight(12)
		sort_number:SetPoint("LEFT", "AllYouCanEat_DataFrame_Sort", "LEFT", 0, 0)
		sort_number:SetScript("OnClick", function() AllYouCanEat:GUIsort(1) end)

		local sort_itemid = CreateFrame("Button", name.."Button_ItemId", button)
		TemplateDataListSortButton(sort_itemid, L["ItemID"], "right", 60)
		sort_itemid:SetWidth(60)
		sort_itemid:SetHeight(12)
		sort_itemid:SetPoint("LEFT", "AllYouCanEat_DataFrame_Sort", "LEFT", 40+2, 0)
		sort_itemid:SetScript("OnClick", function() AllYouCanEat:GUIsort(2) end)

		local sort_itemname = CreateFrame("Button", name.."Button_ItemName", button)
		TemplateDataListSortButton(sort_itemname, L["Name"], "left", 150)
		sort_itemname:SetWidth(220)
		sort_itemname:SetHeight(12)
		sort_itemname:SetPoint("LEFT", "AllYouCanEat_DataFrame_Sort", "LEFT", 100+2+2, 0)
		sort_itemname:SetScript("OnClick", function() AllYouCanEat:GUIsort(3) end)
	else
		local NumberButton = button:CreateFontString(name.."Number", "ARTWORK", "GameFontNormalSmall")
		NumberButton:SetWidth(40)
		NumberButton:SetHeight(12)
		NumberButton:SetPoint("LEFT", button, "LEFT", 0, 0)
		NumberButton:SetJustifyH("RIGHT")

		local ItemIdButton = button:CreateFontString(name.."ItemId", "ARTWORK", "GameFontWhiteSmall")
		ItemIdButton:SetWidth(60)
		ItemIdButton:SetHeight(12)
		ItemIdButton:SetPoint("LEFT", button, "LEFT", 40+2, 0)
		ItemIdButton:SetJustifyH("RIGHT")

		local ItemNameButton = button:CreateFontString(name.."ItemName", "ARTWORK", "GameFontNormalSmall")
		ItemNameButton:SetWidth(220)
		ItemNameButton:SetHeight(12)
		ItemNameButton:SetPoint("LEFT", button, "LEFT", 100+2+2, 0)
		ItemNameButton:SetJustifyH("LEFT")

		local textureHighlight = button:CreateTexture(nil, "ARTWORK")
		textureHighlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		textureHighlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -26, 0)
		textureHighlight:SetTexture(1, 1, 1, 0.2)
		button:SetHighlightTexture(textureHighlight)

		button:SetScript("OnEnter", function() AllYouCanEat:ShowTooltip(button, button.itemID) end)
		button:SetScript("OnLeave", function() GameTooltip:Hide() end)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:CreateMainFrame()
	local mainframe = CreateFrame("Frame", "AllYouCanEat_MainFrame", UIParent)
	mainframe:EnableMouse(true)
	mainframe:SetMovable(true)
	mainframe:SetToplevel(true)
	mainframe:SetClampedToScreen(true)
	mainframe:Hide()
	mainframe:SetWidth(350)
	mainframe:SetHeight(28 + 85 - 2)
	mainframe:SetScript("OnShow", function() AllYouCanEat:MainFrameShow() end)
	mainframe:SetScript("OnHide", function() AllYouCanEat:MainFrameHide() end)

	-- Headline Frame START
	local headline = CreateFrame("Frame", "AllYouCanEat_Headline", mainframe)
	TemplateBorderTRBL(headline)
	headline:SetWidth(350)
	headline:SetHeight(28)
	headline:SetPoint("TOPLEFT", AllYouCanEat_MainFrame, "TOPLEFT", 0, 0)
	headline:EnableMouse(true)
	headline:SetScript("OnMouseDown", function() AllYouCanEat_MainFrame:StartMoving() end)
	headline:SetScript("OnMouseUp", function() AllYouCanEat_MainFrame:StopMovingOrSizing() AllYouCanEat:SaveMainFramePos() end)

	local title = headline:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	title:SetPoint("LEFT", headline, "LEFT", 4, 0)
	title:SetJustifyH("RIGHT")
	title:SetText("AllYouCanEat")

	local headlineButtonX = CreateFrame("Button", "AllYouCanEat_Headline_ButtonX", headline)
	TemplateHeadlineButton(headlineButtonX, nil, 1)
	headlineButtonX:SetWidth(20)
	headlineButtonX:SetHeight(20)
	headlineButtonX:SetPoint("RIGHT", AllYouCanEat_Headline, "RIGHT", -4, 0)
	headlineButtonX:SetScript("OnClick", function() AllYouCanEat:MainFrameToggle() end)

	local headlineButtonCharManager = CreateFrame("Button", "AllYouCanEat_Headline_CharManager", headline)
	TemplateHeadlineButton(headlineButtonCharManager, L["CharManager"], 2)
	headlineButtonCharManager:SetWidth( AllYouCanEat_Headline_CharManagerText:GetWidth() + 10 )
	headlineButtonCharManager:SetHeight(20)
	headlineButtonCharManager:SetPoint("RIGHT", AllYouCanEat_Headline_ButtonX, "LEFT", -4, 0)
	headlineButtonCharManager:SetScript("OnClick", function() AllYouCanEat:CharManagerToggle() end)
--[[
	local headlineButtonSearch = CreateFrame("Button", "AllYouCanEat_Headline_ButtonSearch", headline)
	TemplateHeadlineButton(headlineButtonSearch, nil, 3)
	headlineButtonSearch:SetWidth(20)
	headlineButtonSearch:SetHeight(20)
	headlineButtonSearch:SetPoint("RIGHT", headlineButtonCharManager, "LEFT", -4, 0)
	headlineButtonSearch:SetScript("OnClick", function() AllYouCanEat:SearchFrameToggle() end)
--]]
	-- Headline Frame END

	-- Selection Frame START
	local selection = CreateFrame("Frame", "AllYouCanEat_Selection", mainframe)
	TemplateBorderTRBL(selection)
	selection:SetWidth(350)
	selection:SetHeight(85)
	selection:SetPoint("TOPLEFT", AllYouCanEat_Headline, "BOTTOMLEFT", 0, 1)
	selection:EnableMouse(true)

	for i = 1, #StatisticID do
		local iconbutton = CreateFrame("Button", "AllYouCanEat_Selection_"..StatisticID[i].id, selection)
		iconbutton:SetWidth(32)
		iconbutton:SetHeight(32)
		if i == 1 then
			iconbutton:SetPoint("TOPLEFT", "AllYouCanEat_Selection", "TOPLEFT", 51, -4)
		else
			iconbutton:SetPoint("LEFT", "AllYouCanEat_Selection_"..StatisticID[i-1].id, "RIGHT", 4, 0)
		end
		local icon = iconbutton:CreateTexture("AllYouCanEat_Selection_Icon_"..StatisticID[i].id, "BACKGROUND")
		icon:SetWidth(32)
		icon:SetHeight(32)
		icon:SetAllPoints(iconbutton)
		icon:SetTexture(StatisticID[i].icon)
		local iconborder = iconbutton:CreateTexture("AllYouCanEat_Selection_IconBorder_"..StatisticID[i].id, "ARTWORK")
		iconborder:SetWidth(64)
		iconborder:SetHeight(64)
		iconborder:SetPoint("TOPLEFT", -14, 14)
		iconborder:SetPoint("BOTTOMRIGHT", 14, -14)
		iconborder:SetTexture("Interface\\Buttons\\CheckButtonGlow")
		
		iconbutton:SetScript("OnClick", function()
			GUI_Selection = StatisticID[i].id
			AllYouCanEat:UpdateSelectionButtons()
			AllYouCanEat:UpdateData()
		end)

		iconbutton:SetScript("OnEnter", function()
			_G["AllYouCanEat_Selection_Icon_"..StatisticID[i].id]:SetAlpha(1)
		end)

		iconbutton:SetScript("OnLeave", function()
			if GUI_Selection == StatisticID[i].id then return end
			_G["AllYouCanEat_Selection_Icon_"..StatisticID[i].id]:SetAlpha(0.7)
		end)

		local text1 = iconbutton:CreateFontString("AllYouCanEat_Selection_TextCompleted_"..StatisticID[i].id, "ARTWORK", "GameFontWhiteSmall")
		text1:SetPoint("TOP", icon, "BOTTOM", 0, -2)
		text1:SetJustifyH("CENTER")
		--text1:SetTextColor(0, 0.75, 0, 1)
		local text2 = iconbutton:CreateFontString("AllYouCanEat_Selection_TextTotal_"..StatisticID[i].id, "ARTWORK", "GameFontWhiteSmall")
		text2:SetPoint("TOP", text1, "BOTTOM", 0, -2)
		text2:SetJustifyH("CENTER")
		text2:SetTextColor(0.6, 0.6, 0.6, 1)
	end

	AllYouCanEat:UpdateSelectionButtons()

	local ButtonAll = CreateFrame("Button", "AllYouCanEat_Selection_ButtonAll", selection)
	TemplateTabButton(ButtonAll, L["all"], 1)
	ButtonAll:SetWidth(112)
	ButtonAll:SetHeight(20)
	ButtonAll:SetPoint("BOTTOMLEFT", selection, "BOTTOMLEFT", 4, -1)
	ButtonAll:SetScript("OnClick", function()
		if GUI_Detail == 1 then return end
		GUI_Detail = 1
		AllYouCanEat:UpdateData()
	end)

	local ButtonComplete = CreateFrame("Button", "AllYouCanEat_Selection_ButtonComplete", selection)
	TemplateTabButton(ButtonComplete, L["consumed"], 2)
	ButtonComplete:SetWidth(112)
	ButtonComplete:SetHeight(20)
	ButtonComplete:SetPoint("BOTTOMLEFT", selection, "BOTTOMLEFT", 119, -1)
	ButtonComplete:SetScript("OnClick", function()
		if GUI_Detail == 2 then return end
		GUI_Detail = 2
		AllYouCanEat:UpdateData()
	end)

	local ButtonIncomplete = CreateFrame("Button", "AllYouCanEat_Selection_ButtonIncomplete", selection)
	TemplateTabButton(ButtonIncomplete, L["not consumed"], 3)
	ButtonIncomplete:SetWidth(112)
	ButtonIncomplete:SetHeight(20)
	ButtonIncomplete:SetPoint("BOTTOMLEFT", selection, "BOTTOMLEFT", 234, -1)
	ButtonIncomplete:SetScript("OnClick", function()
		if GUI_Detail == 3 then return end
		GUI_Detail = 3
		AllYouCanEat:UpdateData()
	end)
	-- Selection Frame END

	-- Data Frame START
	local data = CreateFrame("Frame", "AllYouCanEat_DataFrame", mainframe)
	TemplateBorderTRBL(data)
	data:SetWidth(350)
	data:SetHeight(382)
	data:SetPoint("TOPLEFT", AllYouCanEat_Selection, "BOTTOMLEFT", 0, 1)
	data:EnableMouse(true)
	--data:SetToplevel(true)
	--data:Hide()

	-- sort START
	local datasort = CreateFrame("Frame", "AllYouCanEat_DataFrame_Sort", data)
	datasort:SetWidth(350)
	datasort:SetHeight(12)
	datasort:SetPoint("TOPLEFT", data, "TOPLEFT", 4, -4)
	TemplateDataList(datasort, true)
	-- sort END

	-- scrollframe START
	local dataScrollFrame = CreateFrame("ScrollFrame", "AllYouCanEat_DataFrame_ScrollFrame", data, "FauxScrollFrameTemplate")
	dataScrollFrame:SetPoint("TOPLEFT", data, "TOPLEFT", 0, -18)
	dataScrollFrame:SetPoint("BOTTOMRIGHT", data, "BOTTOMRIGHT", -26, 4)
	dataScrollFrame:SetScript("OnVerticalScroll", function() AllYouCanEat:OnVerticalScroll() end)
	AllYouCanEat_DataFrame_ScrollFrameScrollBarThumbTexture:SetWidth(16)
	AllYouCanEat_DataFrame_ScrollFrameScrollBarThumbTexture:SetHeight(16)
	AllYouCanEat_DataFrame_ScrollFrameScrollBarThumbTexture:SetTexCoord(0.21875, 0.75, 0.25, 0.71875) -- (7/32, 24/32, 8/32, 23/32)
	for i = 1, GUI_Data_maxNum do
		local button = CreateFrame("Button", "AllYouCanEat_DataFrame_Button"..i, data)
		TemplateDataList(button)
		button:SetWidth(350)
		button:SetHeight(12)
		if i == 1 then
			button:SetPoint("TOPLEFT", "AllYouCanEat_DataFrame", "TOPLEFT", 4, -18)
		else
			button:SetPoint("TOPLEFT", "AllYouCanEat_DataFrame_Button"..(i-1), "BOTTOMLEFT", 0, 0)
		end
	end
	-- scrollframe END
	-- Data Frame END

	-- Status Frame START
	local status = CreateFrame("Frame", "AllYouCanEat_Status", mainframe)
	TemplateBorderTRBL(status)
	status:SetWidth(350)
	status:SetHeight(20)
	status:SetPoint("TOPLEFT", AllYouCanEat_DataFrame, "BOTTOMLEFT", 0, 1)
	status:EnableMouse(true)

	local text = status:CreateFontString("AllYouCanEat_Status_Text", "ARTWORK", "GameFontWhiteSmall")
	text:SetPoint("CENTER", status, "CENTER", 0, 0)
	-- Status Frame END

	-- CharManager START
	local charmanager = CreateFrame("Frame", "AllYouCanEat_CharManager", mainframe)
	TemplateBorderTRBL(charmanager)
	charmanager:SetWidth(350)
	charmanager:SetHeight(85 + 382 - 1)
	charmanager:SetPoint("TOPLEFT", AllYouCanEat_Headline, "BOTTOMLEFT", 0, 1)
	charmanager:EnableMouse(true)
	charmanager:Hide()

	local realmname = charmanager:CreateFontString("AllYouCanEat_CharManager_Opt_RealmName", "ARTWORK", "GameFontNormal")
	realmname:SetWidth(330)
	realmname:SetHeight(12)
	realmname:SetPoint("TOPLEFT", AllYouCanEat_CharManager, "TOPLEFT", 10, -10)
	realmname:SetJustifyH("LEFT")

	local charlist = CreateFrame("Frame", "AllYouCanEat_CharManager_CharList", charmanager)
	TemplateBorderTRBL(charlist)
	charlist:SetWidth(330)
	charlist:SetHeight(GUI_CharListData_maxNum*12+8)
	charlist:SetPoint("TOPLEFT", AllYouCanEat_CharManager_Opt_RealmName, "BOTTOMLEFT", 0, -10)

	for i = 1, GUI_CharListData_maxNum do
		local button = CreateFrame("Button", "AllYouCanEat_CharManager_CharList_Button"..i, charlist)
		button:SetWidth(330)
		button:SetHeight(12)
		if i == 1 then
			button:SetPoint("TOPLEFT", "AllYouCanEat_CharManager_CharList", "TOPLEFT", 4, -4)
		else
			button:SetPoint("TOPLEFT", "AllYouCanEat_CharManager_CharList_Button"..(i-1), "BOTTOMLEFT", 0, 0)
		end

		local name = button:CreateFontString("AllYouCanEat_CharManager_CharList_Button"..i.."Name", "ARTWORK", "GameFontWhiteSmall")
		name:SetWidth(100)
		name:SetHeight(12)
		name:SetPoint("LEFT", button, "LEFT", 0, 0)
		name:SetJustifyH("LEFT")

		local name = button:CreateFontString("AllYouCanEat_CharManager_CharList_Button"..i.."SaveStats", "ARTWORK", "GameFontWhiteSmall")
		name:SetWidth(115)
		name:SetHeight(12)
		name:SetPoint("LEFT", button, "LEFT", 100, 0)
		name:SetJustifyH("CENTER")

		local name = button:CreateFontString("AllYouCanEat_CharManager_CharList_Button"..i.."Tooltip", "ARTWORK", "GameFontWhiteSmall")
		name:SetWidth(115-8)
		name:SetHeight(12)
		name:SetPoint("LEFT", button, "LEFT", 215, 0)
		name:SetJustifyH("CENTER")

		local texture = button:CreateTexture("AllYouCanEat_CharManager_CharList_Button"..i.."Background", "ARTWORK")
		texture:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		texture:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -8, 0)
		texture:SetTexture(1, 1, 1, 0)

		local textureHighlight = button:CreateTexture(nil, "ARTWORK")
		textureHighlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		textureHighlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -8, 0)
		textureHighlight:SetTexture(1, 1, 1, 0.2)
		button:SetHighlightTexture(textureHighlight)

		button:SetScript("OnClick", function(self) AllYouCanEat:SetupCharacterOptions(self.guid) end)
	end

	local charname = charmanager:CreateFontString("AllYouCanEat_CharManager_Opt_CharName", "ARTWORK", "GameFontNormal")
	charname:SetWidth(330)
	charname:SetHeight(12)
	charname:SetPoint("TOPLEFT", AllYouCanEat_CharManager_CharList, "BOTTOMLEFT", 0, -20)
	charname:SetJustifyH("LEFT")

	local savestatus = charmanager:CreateFontString("AllYouCanEat_CharManager_Opt_SaveStatus_Text", "ARTWORK", "GameFontWhiteSmall")
	savestatus:SetPoint("TOPLEFT", AllYouCanEat_CharManager_Opt_CharName, "BOTTOMLEFT", 0, -10)
	savestatus:SetNonSpaceWrap(true)
	savestatus:SetJustifyH("LEFT")

	CreateFrame("CheckButton", "AllYouCanEat_CharManager_Opt_SaveStats", charmanager, "UIPanelButtonTemplate2")
	AllYouCanEat_CharManager_Opt_SaveStatsText:SetText(L["Save Data"])
	AllYouCanEat_CharManager_Opt_SaveStats:SetWidth(160)
	AllYouCanEat_CharManager_Opt_SaveStats:SetHeight(24)
	AllYouCanEat_CharManager_Opt_SaveStats:SetPoint("TOPLEFT", AllYouCanEat_CharManager_Opt_SaveStatus_Text, "BOTTOMLEFT", 0, -5)
	AllYouCanEat_CharManager_Opt_SaveStats:SetScript("OnClick", function()
		AllYouCanEat:ChangeCharacterOption("savestats")
	end)
	AllYouCanEat_CharManager_Opt_SaveStats:Hide()

	CreateFrame("CheckButton", "AllYouCanEat_CharManager_Opt_DeleteStats", charmanager, "UIPanelButtonTemplate2")
	AllYouCanEat_CharManager_Opt_DeleteStatsText:SetText(L["Delete Data"])
	AllYouCanEat_CharManager_Opt_DeleteStats:SetWidth(160)
	AllYouCanEat_CharManager_Opt_DeleteStats:SetHeight(24)
	AllYouCanEat_CharManager_Opt_DeleteStats:SetPoint("TOPLEFT", AllYouCanEat_CharManager_Opt_SaveStats, "TOPRIGHT", 10, 0)
	AllYouCanEat_CharManager_Opt_DeleteStats:SetScript("OnClick", function()
		AllYouCanEat:ChangeCharacterOption("deletestats")
	end)
	AllYouCanEat_CharManager_Opt_DeleteStats:Hide()

	local text = AllYouCanEat_CharManager_Opt_SaveStats:CreateFontString("AllYouCanEat_CharManager_Opt_SaveStats_Text", "ARTWORK", "GameFontWhiteSmall")
	text:SetPoint("TOPLEFT", AllYouCanEat_CharManager_Opt_SaveStats, "BOTTOMLEFT", 0, -5)
	text:SetWidth(300)
	text:SetNonSpaceWrap(true)
	text:SetJustifyH("LEFT")
	text:SetText(L["Saves the statistics data from this character to SavedVariable, so you can view it with other characters."])

	CreateFrame("CheckButton", "AllYouCanEat_CharManager_Opt_ShowInTooltip", charmanager, "UICheckButtonTemplate")
	TemplateCheckButton(AllYouCanEat_CharManager_Opt_ShowInTooltip)
	AllYouCanEat_CharManager_Opt_ShowInTooltipText:SetText(L["Show statistic data in GameTooltip"])
	AllYouCanEat_CharManager_Opt_ShowInTooltipText:SetPoint("LEFT", AllYouCanEat_CharManager_Opt_ShowInTooltip, "RIGHT", 4, 0)
	AllYouCanEat_CharManager_Opt_ShowInTooltip:SetWidth(14)
	AllYouCanEat_CharManager_Opt_ShowInTooltip:SetHeight(14)
	AllYouCanEat_CharManager_Opt_ShowInTooltip:SetPoint("TOPLEFT", AllYouCanEat_CharManager_Opt_SaveStats_Text, "BOTTOMLEFT", 0, -20)
	AllYouCanEat_CharManager_Opt_ShowInTooltip:SetScript("OnClick", function()
		AllYouCanEat:ChangeCharacterOption("tooltip")
	end)
	AllYouCanEat_CharManager_Opt_ShowInTooltip:Hide()
	-- CharManager END
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:ChangeCharacterOption(what)
	local guid = AllYouCanEat_CharManager_Opt_CharName.guid

	if what == "savestats" then
		AllYouCanEat:SaveCharacterStatisticData()
	elseif what == "deletestats" then
		AllYouCanEat:DeleteCharacterStatisticData(guid)
	elseif what == "tooltip" then
		for playerrealm in pairs(AllYouCanEat_Data.chars) do
			for playerguid, data in pairs(AllYouCanEat_Data.chars[playerrealm]) do
				if playerguid == guid then
					data.tooltip = not data.tooltip
					break
				end
			end
		end
		AllYouCanEat:CheckIfTooltipCodeIsNeeded()
	end
	AllYouCanEat:SetupCharacterOptions(guid)
end

function AllYouCanEat:SetupCharacterOptions(guid)
	local name = ""
	local class = "PRIEST"
	local tooltip = false
	local savestats = false

	for playerrealm in pairs(AllYouCanEat_Data.chars) do
		for playerguid, data in pairs(AllYouCanEat_Data.chars[playerrealm]) do
			if playerguid == guid then
				name = data.name
				class = data.class
				if data.tooltip then
					tooltip = true
				end
				if data.savestats then
					savestats = true
				end
				break
			end
		end
	end

	-- realmname START
	AllYouCanEat_CharManager_Opt_RealmName:SetText(playerRealm)
	-- realmname END

	-- charname START
	AllYouCanEat_CharManager_Opt_CharName:SetText(L["Options for:"].." |cff"..ClassHexColor(class)..name.."|r")
	AllYouCanEat_CharManager_Opt_CharName.guid = guid
	-- charname END

	-- save status START
	if savestats then
		AllYouCanEat_CharManager_Opt_SaveStatus_Text:SetText(L["Status:"].." "..L["SAVED"])
		AllYouCanEat_CharManager_Opt_SaveStatus_Text:SetTextColor(0, 0.75, 0, 1)
	else
		AllYouCanEat_CharManager_Opt_SaveStatus_Text:SetText(L["Status:"].." "..L["NOT SAVED"])
		AllYouCanEat_CharManager_Opt_SaveStatus_Text:SetTextColor(1, 0, 0, 1)
	end
	-- save status END

	-- save/delete button START
	if playerGUID == guid then
		if savestats then
			AllYouCanEat_CharManager_Opt_SaveStats:Disable()
			AllYouCanEat_CharManager_Opt_DeleteStats:Enable()
		else
			AllYouCanEat_CharManager_Opt_SaveStats:Enable()
			AllYouCanEat_CharManager_Opt_DeleteStats:Disable()
		end
		AllYouCanEat_CharManager_Opt_SaveStats_Text:SetTextColor(1, 1, 1, 1)
	else
		if savestats then
			AllYouCanEat_CharManager_Opt_SaveStats:Disable()
			AllYouCanEat_CharManager_Opt_DeleteStats:Enable()
		else
			AllYouCanEat_CharManager_Opt_SaveStats:Disable()
			AllYouCanEat_CharManager_Opt_DeleteStats:Disable()
		end
		AllYouCanEat_CharManager_Opt_SaveStats_Text:SetTextColor(0.3, 0.3, 0.3, 1)
	end
	AllYouCanEat_CharManager_Opt_SaveStats:Show()
	AllYouCanEat_CharManager_Opt_DeleteStats:Show()
	-- save/delete button END

	-- tooltip START
	if savestats then
		AllYouCanEat_CharManager_Opt_ShowInTooltip:Enable()
		AllYouCanEat_CharManager_Opt_ShowInTooltipText:SetTextColor(1, 1, 1, 1)
	else
		AllYouCanEat_CharManager_Opt_ShowInTooltip:Disable()
		AllYouCanEat_CharManager_Opt_ShowInTooltipText:SetTextColor(0.3, 0.3, 0.3, 1)
	end
	AllYouCanEat_CharManager_Opt_ShowInTooltip:Show()
	AllYouCanEat_CharManager_Opt_ShowInTooltip:SetChecked(tooltip)
	-- tooltip END
	AllYouCanEat:UpdateData_CharList()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:UpdateData_CharList()
	table.wipe(GUI_CharListData)

	local x = 1
	for k, v in pairs(AllYouCanEat_Data.chars[playerRealm]) do
		GUI_CharListData[x] = {}
		GUI_CharListData[x].name = v.name
		GUI_CharListData[x].class = v.class
		if v.savestats then
			GUI_CharListData[x].savestats = "|cff00bf00"..L["SAVED"].."|r"
		else
			GUI_CharListData[x].savestats = "|cffff0000"..L["NOT SAVED"].."|r"
		end
		if v.tooltip then
			GUI_CharListData[x].tooltip = L["TOOLTIP"]
		end
		GUI_CharListData[x].guid = k
		x = x + 1
	end

	table.sort(GUI_CharListData, function(a, b)
		if a.name < b.name then
			return true
		end
	end)

	for i = 1, GUI_CharListData_maxNum do
		_G["AllYouCanEat_CharManager_CharList_Button"..i]:Hide()
	end

	for i = 1, #GUI_CharListData do
		if GUI_CharListData[i].guid == AllYouCanEat_CharManager_Opt_CharName.guid then
			_G["AllYouCanEat_CharManager_CharList_Button"..i.."Background"]:SetTexture(1, 1, 1, 0.1)
		else
			_G["AllYouCanEat_CharManager_CharList_Button"..i.."Background"]:SetTexture(1, 1, 1, 0)
		end

		_G["AllYouCanEat_CharManager_CharList_Button"..i.."Name"]:SetText("|cff"..ClassHexColor(GUI_CharListData[i].class)..GUI_CharListData[i].name.."|r")
		_G["AllYouCanEat_CharManager_CharList_Button"..i.."SaveStats"]:SetText(GUI_CharListData[i].savestats)
		_G["AllYouCanEat_CharManager_CharList_Button"..i.."Tooltip"]:SetText(GUI_CharListData[i].tooltip)
		_G["AllYouCanEat_CharManager_CharList_Button"..i]:Show()
		_G["AllYouCanEat_CharManager_CharList_Button"..i].guid = GUI_CharListData[i].guid
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:InitializeFrame()
	AllYouCanEat:UpdateSortButtons()
end

function AllYouCanEat:MainFrameToggle()
	if AllYouCanEat_MainFrame:IsShown() then
		AllYouCanEat_MainFrame:Hide()
	else
		AllYouCanEat_MainFrame:Show()
		AllYouCanEat:CharManagerToggle(true)
	end
end

function AllYouCanEat:CharManagerToggle(force)
	if AllYouCanEat_CharManager:IsShown() or force then
		AllYouCanEat_CharManager:Hide()
		AllYouCanEat_Headline_CharManagerBackground:SetTexture(1, 1, 1, 0)
		AllYouCanEat_Selection:Show()
		AllYouCanEat_DataFrame:Show()
	else
		AllYouCanEat_CharManager:Show()
		AllYouCanEat:SetupCharacterOptions(playerGUID)
		AllYouCanEat_Headline_CharManagerBackground:SetTexture(0.6, 0.6, 0.6, 0.5)
		AllYouCanEat_Selection:Hide()
		AllYouCanEat_DataFrame:Hide()
	end
end

function AllYouCanEat:MainFramePos()
	if AllYouCanEat_Options.MainFrame_relP then
		AllYouCanEat_MainFrame:ClearAllPoints()
		AllYouCanEat_MainFrame:SetPoint(AllYouCanEat_Options.MainFrame_relP, AllYouCanEat_Options.MainFrame_posX, AllYouCanEat_Options.MainFrame_posY)
	else
		AllYouCanEat_MainFrame:ClearAllPoints()
		AllYouCanEat_MainFrame:SetPoint("CENTER", 0, 300)
	end
end

function AllYouCanEat:MainFrameHide()
	PlaySound("igQuestListClose")
	AllYouCanEat_InterfaceOptions_GUI:Enable()
end

function AllYouCanEat:MainFrameShow()
	PlaySound("igQuestListOpen")
	AllYouCanEat_InterfaceOptions_GUI:Disable()
	AllYouCanEat:MainFramePos()
	AllYouCanEat:UpdateData()
end

function AllYouCanEat:SaveMainFramePos()
	local point, relativeTo, relativePoint, x, y = AllYouCanEat_MainFrame:GetPoint()
	AllYouCanEat_Options.MainFrame_posX = x
	AllYouCanEat_Options.MainFrame_posY = y
	AllYouCanEat_Options.MainFrame_relP = relativePoint
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AllYouCanEat:UpdateData()
	AllYouCanEat:CreateGUIdata()
	AllYouCanEat:FillList()
	AllYouCanEat:UpdateStatus()

	if GUI_Detail == 1 then
		SetTabButton("AllYouCanEat_Selection_ButtonAll", true)
		SetTabButton("AllYouCanEat_Selection_ButtonComplete", nil)
		SetTabButton("AllYouCanEat_Selection_ButtonIncomplete", nil)
	elseif GUI_Detail == 2 then
		SetTabButton("AllYouCanEat_Selection_ButtonAll", nil)
		SetTabButton("AllYouCanEat_Selection_ButtonComplete", true)
		SetTabButton("AllYouCanEat_Selection_ButtonIncomplete", nil)
	elseif GUI_Detail == 3 then
		SetTabButton("AllYouCanEat_Selection_ButtonAll", nil)
		SetTabButton("AllYouCanEat_Selection_ButtonComplete", nil)
		SetTabButton("AllYouCanEat_Selection_ButtonIncomplete", true)
	end
end

function AllYouCanEat:UpdateSelectionButtons()
	for i = 1, #StatisticID do
		if GUI_Selection == StatisticID[i].id then
			_G["AllYouCanEat_Selection_IconBorder_"..StatisticID[i].id]:Show()
			_G["AllYouCanEat_Selection_Icon_"..StatisticID[i].id]:SetAlpha(1)
		else
			_G["AllYouCanEat_Selection_IconBorder_"..StatisticID[i].id]:Hide()
			_G["AllYouCanEat_Selection_Icon_"..StatisticID[i].id]:SetAlpha(0.7)
		end
	end
end

function AllYouCanEat:UpdateStatus()
	AllYouCanEat_Status_Text:SetText(L["Latest Update"]..": "..GUI_scantime)
	for i = 1, #StatisticID do
		_G["AllYouCanEat_Selection_TextCompleted_"..StatisticID[i].id]:SetText(DATA[StatisticID[i].id].totalcompleted)
		_G["AllYouCanEat_Selection_TextTotal_"..StatisticID[i].id]:SetText("("..DATA[StatisticID[i].id].total..")")
	end
end

function AllYouCanEat:CreateGUIdata()
	table.wipe(GUI_Data)

	if GUI_Detail == 1 then
		local x = 1
		for k, v in pairs(DATA[GUI_Selection]) do
			if k ~= "total" and k ~= "totalcompleted" then
				GUI_Data[x] = {}
				GUI_Data[x].itemnumber = x
				GUI_Data[x].itemid = k
				GUI_Data[x].itemname = v.itemname
				GUI_Data[x].completed = v.completed
				x = x + 1
			end
		end
	elseif GUI_Detail == 2 then
		local x = 1
		for k, v in pairs(DATA[GUI_Selection]) do
			if k ~= "total" and k ~= "totalcompleted" and v.completed then
				GUI_Data[x] = {}
				GUI_Data[x].itemnumber = x
				GUI_Data[x].itemid = k
				GUI_Data[x].itemname = v.itemname
				GUI_Data[x].completed = v.completed
				x = x + 1
			end
		end
	elseif GUI_Detail == 3 then
		local x = 1
		for k, v in pairs(DATA[GUI_Selection]) do
			if k ~= "total" and k ~= "totalcompleted" and not v.completed then
				GUI_Data[x] = {}
				GUI_Data[x].itemnumber = x
				GUI_Data[x].itemid = k
				GUI_Data[x].itemname = v.itemname
				GUI_Data[x].completed = v.completed
				x = x + 1
			end
		end
	end

	GUI_Data_Count = #GUI_Data

	local sortfunc
	if GUI_SortBy == 1 then -- 1 = number
		if GUI_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.itemnumber < b.itemnumber then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.itemnumber > b.itemnumber then
					return true
				end
			end
		end
	elseif GUI_SortBy == 2 then -- 2 = itemid
		if GUI_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.itemid < b.itemid then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.itemid > b.itemid then
					return true
				end
			end
		end
	elseif GUI_SortBy == 3 then -- 3 = itemname
		if GUI_SortOrder == 1 then
			sortfunc = function(a, b)
				if a.itemname < b.itemname then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.itemname > b.itemname then
					return true
				end
			end
		end
	end

	table.sort(GUI_Data, sortfunc)
end

function AllYouCanEat:GUIsort(sortBy)
	if GUI_SortBy == sortBy then
		if GUI_SortOrder == 1 then
			GUI_SortOrder = 0
		else
			GUI_SortOrder = 1
		end
	else
		GUI_SortOrder = 1
	end
	GUI_SortBy = sortBy

	AllYouCanEat:UpdateSortButtons()
	AllYouCanEat:CreateGUIdata()
	AllYouCanEat:FillList()
end

function AllYouCanEat:UpdateSortButtons()
	local arrow
	for i = 1, 3 do
		if i == 1 then     -- 1 = Number
			arrow = "AllYouCanEat_DataFrame_SortButton_NumberArrow"
		elseif i == 2 then -- 2 = ItemId
			arrow = "AllYouCanEat_DataFrame_SortButton_ItemIdArrow"
		elseif i == 3 then -- 3 = ItemName
			arrow = "AllYouCanEat_DataFrame_SortButton_ItemNameArrow"
		end
		if GUI_SortBy == i then
			if GUI_SortOrder == 1 then _G[arrow]:SetTexCoord(0, 0.5625, 0, 1) end
			if GUI_SortOrder == 0 then _G[arrow]:SetTexCoord(0.5625, 0, 1, 0) end
		else
			_G[arrow]:SetTexCoord(0, 0, 0, 0)
		end
	end
end

function AllYouCanEat:OnVerticalScroll()
	AllYouCanEat_DataFrame_ScrollFrameScrollBar:SetValue(arg1)
	AllYouCanEat_DataFrame_ScrollFrame.offset = math.floor((arg1 / 12) + 0.5)
	AllYouCanEat:FillList()
end

function AllYouCanEat:FillList()
	local showtableNumSize = GUI_Data_maxNum

	if GUI_Data_Count < showtableNumSize then
		for i=1, showtableNumSize do
			_G["AllYouCanEat_DataFrame_Button"..i]:Hide()
		end
		showtableNumSize = GUI_Data_Count
	end

	FauxScrollFrame_Update(AllYouCanEat_DataFrame_ScrollFrame, GUI_Data_Count, showtableNumSize, 12)

	AllYouCanEat_DataFrame_ScrollFrame:Show()
	if GUI_Data_Count <= GUI_Data_maxNum then
		AllYouCanEat_DataFrame_ScrollFrame:SetAlpha(0.2)
		AllYouCanEat_DataFrame_ScrollFrameScrollBarScrollUpButton:Disable()
		AllYouCanEat_DataFrame_ScrollFrameScrollBarScrollDownButton:Disable()
	else
		AllYouCanEat_DataFrame_ScrollFrame:SetAlpha(1)
	end

	local j = 1
	local sliderPos = AllYouCanEat_DataFrame_ScrollFrame.offset
	for i=sliderPos+1, sliderPos+showtableNumSize do
		if sliderPos < GUI_Data_Count then
			_G["AllYouCanEat_DataFrame_Button"..j.."Number"]:SetText(GUI_Data[i].itemnumber)
			_G["AllYouCanEat_DataFrame_Button"..j.."ItemId"]:SetText(GUI_Data[i].itemid)
			_G["AllYouCanEat_DataFrame_Button"..j.."ItemName"]:SetText(GUI_Data[i].itemname)

			if GUI_Data[i].completed then
				_G["AllYouCanEat_DataFrame_Button"..j.."ItemName"]:SetTextColor(0, 0.75, 0, 1)
			else
				_G["AllYouCanEat_DataFrame_Button"..j.."ItemName"]:SetTextColor(1, 0, 0, 1)
			end

			_G["AllYouCanEat_DataFrame_Button"..j]:Show()
			_G["AllYouCanEat_DataFrame_Button"..j].itemID = GUI_Data[i].itemid
			j = j + 1
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function AllYouCanEat_Hook_UseContainerItem(bag, slot)
	--local timerStart = GetTime() -- timecheck
	if bag  and type(bag) ~= "number" then return end
	if slot and type(slot) ~= "number" then return end
	if bag == BANK_CONTAINER then return end

	local itemID = GetContainerItemID(bag, slot)

	if not itemID then return end

	for i = 1, #StatisticID do
		local statID = StatisticID[i].id
		if DATA[statID][itemID] and not DATA[statID][itemID].completed then
			UPDATE_itemID = itemID
			UPDATE_statID = statID
			UPDATE_catID  = i
			criteriaTimer = GetTime()
			return
		end
	end
	--print("timecheck:", GetTime()-timerStart) --timecheck
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function OnEvent(self, event, ...)
	if event == "CRITERIA_UPDATE" then
		if not UPDATE_itemID then return end -- <- !!! muy importante, amigo !!!

		local _, _, isCompleted = GetAchievementCriteriaInfo(DATA[UPDATE_statID][UPDATE_itemID].criteriaID)

		if isCompleted then
			DATA[UPDATE_statID][UPDATE_itemID].completed = true
			DATA[UPDATE_statID].totalcompleted = DATA[UPDATE_statID].totalcompleted + 1

			if AllYouCanEat_Data.stats and
			   AllYouCanEat_Data.stats[UPDATE_statID] and
			   AllYouCanEat_Data.stats[UPDATE_statID][UPDATE_itemID] and
			   AllYouCanEat_Data.stats[UPDATE_statID][UPDATE_itemID][playerGUID]
			then
				AllYouCanEat_Data.stats[UPDATE_statID][UPDATE_itemID][playerGUID] = nil
				local x = 0
				for k, v in pairs(AllYouCanEat_Data.stats[UPDATE_statID][UPDATE_itemID]) do
					x = x + 1
					break
				end
				if x == 0 then
					AllYouCanEat_Data.stats[UPDATE_statID][UPDATE_itemID] = nil
				end
			end

			local _, itemLink = GetItemInfo(UPDATE_itemID)
			if itemLink then
				print("|cffffff7fAllYouCanEat:|r", "|cffff9900"..NEW.."|r", itemLink, "- |cffcccccc("..StatisticID[UPDATE_catID].name.." | "..L["ItemID"]..": "..UPDATE_itemID..")|r")
			else
				print("|cffffff7fAllYouCanEat:|r", "|cffff9900"..NEW.."|r", DATA[UPDATE_statID][UPDATE_itemID].itemname, "- |cffcccccc("..StatisticID[UPDATE_catID].name.." | "..L["ItemID"]..": "..UPDATE_itemID..")|r")
			end

			-- if UPDATE_itemID exists more than one time START
			for i = 1, #StatisticID do
				local statID = StatisticID[i].id
				if DATA[statID][UPDATE_itemID] and not DATA[statID][UPDATE_itemID].completed then

					DATA[statID][UPDATE_itemID].completed = true
					DATA[statID].totalcompleted = DATA[statID].totalcompleted + 1

					if AllYouCanEat_Data.stats and
					   AllYouCanEat_Data.stats[statID] and
					   AllYouCanEat_Data.stats[statID][UPDATE_itemID] and
					   AllYouCanEat_Data.stats[statID][UPDATE_itemID][playerGUID]
					then
						AllYouCanEat_Data.stats[statID][UPDATE_itemID][playerGUID] = nil
						local x = 0
						for k, v in pairs(AllYouCanEat_Data.stats[statID][UPDATE_itemID]) do
							x = x + 1
							break
						end
						if x == 0 then
							AllYouCanEat_Data.stats[statID][UPDATE_itemID] = nil
						end
					end

					if itemLink then
						print("|cffffff7fAllYouCanEat:|r", "|cffff9900"..NEW.."|r", itemLink, "- |cffcccccc("..StatisticID[i].name.." | "..L["ItemID"]..": "..UPDATE_itemID..")|r")
					else
						print("|cffffff7fAllYouCanEat:|r", "|cffff9900"..NEW.."|r", DATA[statID][UPDATE_itemID].itemname, "- |cffcccccc("..StatisticID[i].name.." | "..L["ItemID"]..": "..UPDATE_itemID..")|r")
					end

				end
			end
			-- if UPDATE_itemID exists more than one time END

			UpdateTime()
			if AllYouCanEat_MainFrame:IsShown() then
				AllYouCanEat:UpdateData()
			end

			UPDATE_itemID = nil

		else
			-- the duration between a sucessful item click (with id from initial stats scan) and CRITERIA_UPDATE action is set to 5 seconds
			if GetTime() - criteriaTimer > 5 then
				UPDATE_itemID = nil
			end
		end
	elseif event == "PLAYER_LOGIN" then
		playerRealm   = GetCVar("realmName")
		playerName    = UnitName("player")
		playerGUID    = UnitGUID("player")
		playerFaction = UnitFactionGroup("player")
		playerClass   = select(2, UnitClass("player"))

		AllYouCanEat:SaveCharacterData()
		AllYouCanEat:InitialStatisticScan()
		AllYouCanEat:InitOptions()
		AllYouCanEat:CreateInterfaceOptions()
		AllYouCanEat:LDBcheck()
		AllYouCanEat:CreateMainFrame()
		AllYouCanEat:InitializeFrame()
		table.insert(UISpecialFrames, "AllYouCanEat_MainFrame")

		AllYouCanEat:CheckIfTooltipCodeIsNeeded()

		InstallHooks(_G.GameTooltip, Hooks)
		InstallHooks(_G.ItemRefTooltip, Hooks)

		hooksecurefunc("UseContainerItem", AllYouCanEat_Hook_UseContainerItem)

		AllYouCanEat:UnregisterEvent("PLAYER_LOGIN")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

AllYouCanEat:RegisterEvent("CRITERIA_UPDATE")
AllYouCanEat:RegisterEvent("PLAYER_LOGIN")
AllYouCanEat:SetScript("OnEvent", OnEvent)